/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2014 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.glassfish.hk2.configuration.hub.xml.dom.integration.tests;

import java.beans.PropertyVetoException;
import java.util.List;

import org.jvnet.hk2.config.ConfigBeanProxy;
import org.jvnet.hk2.config.ConfigSupport;
import org.jvnet.hk2.config.Configured;
import org.jvnet.hk2.config.DuckTyped;
import org.jvnet.hk2.config.Element;
import org.jvnet.hk2.config.SingleConfigCode;
import org.jvnet.hk2.config.TransactionFailure;

/**
 * @author jwells
 *
 */
@Configured
public interface HBean extends ConfigBeanProxy {
    @Element("*")
    public List<DBean> getDBeans();
    
    @DuckTyped
    public <T extends DBean> T createDBean(String name);
    
    @DuckTyped
    public <T extends DBean> T getDBeanByName(String name);
    
    @DuckTyped
    public <T extends DBean> T deleteDBean(T removeMe);
    
    class Duck {
        public static <T extends DBean> T createDBean(final HBean hbean, final String name) throws TransactionFailure {
            ConfigSupport.apply(new SingleConfigCode<HBean>() {

                @Override
                public Object run(HBean writeableHBean) throws PropertyVetoException,
                        TransactionFailure {
                    DBean dbean = writeableHBean.createChild(DBean.class);
                    dbean.setName(name);
                    
                    writeableHBean.getDBeans().add(dbean);
                    
                    return dbean;
                }}, hbean);
            
            return hbean.getDBeanByName(name);
        }
        
        @SuppressWarnings("unchecked")
        public static <T extends DBean> T deleteDBean(final HBean hbean, final T removeMe) throws TransactionFailure {
            return (T) ConfigSupport.apply(new SingleConfigCode<HBean>() {

                @Override
                public Object run(HBean writeableHBean)
                        throws TransactionFailure {
                    writeableHBean.getDBeans().remove(removeMe);
                    return removeMe;
                }

            }, hbean);
            
        }
        
        @SuppressWarnings("unchecked")
        public static <T extends DBean> T getDBeanByName(final HBean hbean, String name) throws TransactionFailure {
            if (name == null) return null;
            if (hbean.getDBeans() == null) return null;
            
            for (DBean dbean : hbean.getDBeans()) {
                if (dbean.getName().equals(name)) return (T) dbean;
            }
            
            return null;
            
            
        }
        
    }

}
