package org.glassfish.hk2.classmodel.reflect.test.parameterized;

/**
 * Created by IntelliJ IDEA.
 * User: dochez
 * Date: 6/29/11
 * Time: 2:34 PM
 * To change this template use File | Settings | File Templates.
 */
public interface Pattern {
    String patternTest();
}
