/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2012 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.glassfish.hk2.tests.locator.lifecycle;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.inject.Singleton;

import org.glassfish.hk2.api.Filter;
import org.glassfish.hk2.api.InstanceLifecycleEvent;
import org.glassfish.hk2.api.InstanceLifecycleListener;
import org.glassfish.hk2.utilities.BuilderHelper;

/**
 * @author jwells
 *
 */
@Singleton
public class InstanceLifecycleListenerImpl implements InstanceLifecycleListener {
    private final Filter notifyeeFilter = BuilderHelper.createContractFilter(Notifyee.class.getName());
    private final Map<Notifyee, Set<Notifier>> disposePath = new HashMap<Notifyee, Set<Notifier>>();

    /* (non-Javadoc)
     * @see org.glassfish.hk2.api.InstanceLifecycleListener#getFilter()
     */
    @Override
    public Filter getFilter() {
        return notifyeeFilter;
    }
    
    private void preDestruction(InstanceLifecycleEvent lifecycleEvent) {
        Notifyee notifyee = (Notifyee) lifecycleEvent.getLifecycleObject();
        Set<Notifier> forDisposal = disposePath.get(notifyee);
        if (forDisposal == null) return;
        
        for (Notifier notifier : forDisposal) {
            notifier.removeNotifyee(notifyee);
            
        }
    }
    
    private void postProduction(InstanceLifecycleEvent lifecycleEvent) {
        if (lifecycleEvent.getKnownInjectees() == null) {
            return;
        }
        
        for (Object injectedInstance : lifecycleEvent.getKnownInjectees().values()) {
            if (!(injectedInstance instanceof Notifier)) {
                continue;
            }
            
            Notifier notifier = (Notifier) injectedInstance;
            Notifyee notifyee = (Notifyee) lifecycleEvent.getLifecycleObject();
            
            Set<Notifier> forDisposal = disposePath.get(notifyee);
            if (forDisposal == null) {
                forDisposal = new HashSet<Notifier>();
                disposePath.put(notifyee, forDisposal);
            }
            forDisposal.add(notifier);
            
            notifier.addNotifyee(notifyee);
        }
        
    }

    /* (non-Javadoc)
     * @see org.glassfish.hk2.api.InstanceLifecycleListener#lifecycleEvent(org.glassfish.hk2.api.InstanceLifecycleEvent)
     */
    @Override
    public void lifecycleEvent(InstanceLifecycleEvent lifecycleEvent) {
        switch (lifecycleEvent.getEventType()) {
        case POST_PRODUCTION:
            postProduction(lifecycleEvent);
            return;
        case PRE_DESTRUCTION:
            preDestruction(lifecycleEvent);
            return;
         default:
            return;
        }
    }

}
