/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2015 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.jvnet.hk2.generator.internal;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.List;
import java.util.TreeSet;

import org.glassfish.hk2.utilities.DescriptorImpl;
import org.jvnet.hk2.generator.InFlightGenerator;

/**
 * @author jwells
 *
 */
public class InFlightGeneratorImpl implements InFlightGenerator {

    /* (non-Javadoc)
     * @see org.jvnet.hk2.generator.InFlightGenerator#generateFromMultipleDirectories(java.util.List, java.util.List, boolean, java.io.OutputStream)
     */
    @Override
    public void generateFromMultipleDirectories(List<File> directories,
            List<File> searchPath, boolean verbose, OutputStream inhabitantFile)
            throws IOException {
        Utilities utilities = new Utilities(verbose, searchPath);
        
        TreeSet<DescriptorImpl> retVal = new TreeSet<DescriptorImpl>(new DescriptorComparitor());
        for (File directory : directories) {
            if (!directory.exists()) continue;
            
            if (!directory.isDirectory()) {
                throw new IllegalArgumentException(directory.getAbsolutePath() + " is not a directory");
            }
            
            List<DescriptorImpl> newOnes = utilities.findAllServicesFromDirectory(directory, directories);
            if (newOnes != null) {
                retVal.addAll(newOnes);
            }
        }
        
        utilities.close();
        
        if (retVal.isEmpty()) return;
        
        PrintWriter writer = new PrintWriter(inhabitantFile);
        for (DescriptorImpl di : retVal) {
            di.writeObject(writer);
        }
        
        writer.flush();
    }

}
