package org.glassfish.hk2.bootstrap;

/**
 * @deprecated use {@link org.glassfish.hk2.api.DescriptorFileFinder} instead
 * @author jwells
 *
 */
public interface DescriptorFileFinder extends org.glassfish.hk2.api.DescriptorFileFinder {
}
